#ifndef _SHOPSMART_H_
#define _SHOPSMART_H_

#include <FApp.h>
#include <FBase.h>
#include <FSystem.h>
#include <FUi.h>
#include <FIo.h>
#include "shop.h"
#include "HUNStringComparer.h"
#include "NavigationForm.h"

/**
 * [ShopSmart] application must inherit from Application class
 * which provides basic features necessary to define an application.
 */
class ShopSmart :
	public Osp::App::Application,
	public Osp::System::IScreenEventListener,
	public Osp::Ui::IActionEventListener,
	public Osp::Ui::IItemEventListener
{
public:

	/**
	 * [ShopSmart] application must have a factory method that creates an instance of itself.
	 */
	static Osp::App::Application* CreateInstance(void);
	virtual void  OnActionPerformed (const Osp::Ui::Control &source, int actionId);
	virtual void  OnItemStateChanged(const Osp::Ui::Control &source, int index, int itemId, Osp::Ui::ItemStatus status);

	// Action Ids
	static const int ID_MAINFORM_BTN1 = 10001;
	static const int ID_MAINFORM_BTN2 = 10002;
	static const int ID_MAINFORM_BTN3 = 10003;
	static const int ID_MAINFORM_BTN4 = 10004;

	static const int ID_FILESELECTFORM_BTN1 = 20001;
	static const int ID_FILESELECTFORM_BTN2 = 20002;

	static const int ID_SAVEFILE_BTNOK = 30001;
	static const int ID_SAVEFILE_BTNC = 30002;

	static const int ID_ISF_SOFTMENU = 40000;
	static const int ID_ISF_SOFTPLAN = 40001;
	static const int ID_ISF_OPT = 40002;
	static const int ID_ISF_OPT_LOAD_CART = 40003;
	static const int ID_ISF_OPT_SAVE_CART = 40004;
	static const int ID_ISF_OPT_MOVE_ITEM = 40005;
	static const int ID_ISF_OPT_NEW_ITEM = 40006;
	static const int ID_ISF_OPT_SAVE_SHOP = 40007;
	static const int ID_ISF_OPT_DELETE_SHOP = 40008;
	static const int ID_ISF_OPT_DELETE_CART = 40009;
	static const int ID_ISF_OPT_DELETE_ITEM = 40010;
	static const int ID_ISF_BTEL = 40020;

	static const int ID_CF_SOFTCART = 50001;
	static const int ID_CF_SOFTSTART = 50002;

	static const int ID_NF_SOFTCART = 60001;
	static const int ID_NF_SOFTNEXT = 60002;

	static const int ID_SS_MENU = 70001;

	static const int ID_IS_SELECT = 80001;

	static const int ID_NI_OK = 90001;
	static const int ID_NI_POPUP_CANCEL = 90002;



public:
	ShopSmart();
	~ShopSmart();
private:
	Osp::Ui::Controls::Frame *__pFrame;
	result SwitchToForm(Osp::Ui::Controls::Form* pNextForm);
	Osp::Ui::Controls::Form *__pCURRF, *__pNEXTF, *__pMainForm, *__pFileSelectForm, *__pSaveForm;
	Osp::Ui::Controls::Form *__pItemSelectForm, *__pCalculatedForm, *__pStatisticsForm, *__pItemSelectorForm;
	Osp::Ui::Controls::Popup *__pNewItemPopup;
	NavigationForm* __pNavigationForm;
	Osp::Ui::Controls::List* __pFileList;
	Osp::Ui::Controls::List* __pSaveFileList;
	Osp::Ui::Controls::EditField* __pSaveFileEdit, *__pNewItemField;
	Osp::Ui::Controls::List* __pItemSelectList, *__pItemSelectorList;
	Osp::Ui::Controls::Button* __pSelectedItemsButton;
	Osp::Ui::Controls::OptionMenu* __pISFOptionMenu;
	Osp::Ui::Controls::Label* __pISFCartLabel, *__pISFLabel, *__pCFShopLabel, *__pCFAddressLabel, *__pCFDistanceLabel, *__pCFDurationLabel, *__pCFArrivalLabel;
	Osp::Ui::Controls::Label* __pSSDurationLabel, *__pSSSpeedLabel, *__pSSCalorieLabel;

	void FillShopList(void);
	void FillShopSaveList(void);
	void FillItemSelectList(void);
	void FillCartSaveList(void);
	void FillCartList(void);
	result SaveCart(File* Fajl);
	result LoadCart(File* Fajl, LinkedList* NotInShopListN);

	//**********LOGICAL SECTION******************************************************
	shop* __pShop;
	String dirName, itemName;
	char mode;

	Osp::Base::Collection::LinkedList Cart;
	String MoveWhat, MoveWhere, SelectedItem;

	Osp::Base::Collection::HashMap* __pFileListItems;
	Osp::Base::Collection::HashMap* __pSaveFileListItems;
	Osp::Base::Collection::HashMap* __pSelectItemsListItems;


public:


	// Called when the application is initializing.
	bool OnAppInitializing(Osp::App::AppRegistry& appRegistry);

	// Called when the application is terminating.
	bool OnAppTerminating(Osp::App::AppRegistry& appRegistry, bool forcedTermination = false);


	// Called when the application's frame moves to the top of the screen.
	void OnForeground(void);


	// Called when this application's frame is moved from top of the screen to the background.
	void OnBackground(void);

	// Called when the system memory is not sufficient to run the application any further.
	void OnLowMemory(void);

	// Called when the battery level changes.
	void OnBatteryLevelChanged(Osp::System::BatteryLevel batteryLevel);

	// Called when the screen turns on.
	void OnScreenOn (void);

	// Called when the screen turns off.
	void OnScreenOff (void);
};

#endif
